import os
from datetime import datetime
import shutil
import numpy as np
from matplotlib import pyplot as plt
from pandas import DataFrame
import matplotlib.lines as mlines
import pandas
import matplotlib
from netCDF4 import Dataset

mode = 'obs'

os.chdir('/esarchive/scratch/mfalls/pisces1d/logs/O5_LAB1_new')

def make_grid(n):
    y=1
    x=1
    while y*x < n:
        if y==x:
            y=y+1
        else:
            x=x+1
    j=0
    i=0
    tuples=[]
    for a in range(n):
        tuples.append((j,i))
        if i == x - 1:
            i=0
            j=j+1
        else:
            i=i+1

    return y,x,tuples

expids=['a3nf','a3ng','a3nh','a3ni','a3nj']
runs = expids
templatepath = '/gpfs/scratch/bsc32/bsc32786/pisces1d/parameters/variables5wchld.csv'

basepath=os.getcwd() + '/'
paraspath='/parameters/'
bbp700path='/bbp700/'
modelfile='PISCES_5d_19600101_19691231_ptrc_T.nc'
obsfile='/gpfs/scratch/bsc32/bsc32786/pisces1d/obs/Mprof_6901486_L75_5days_2015-01-01_2015-12-31reg.nc'

## get breakpoints
bpfile = '/esarchive/scratch/mfalls/pisces1d/logs/O5_LAB3/LAB3_21-05-17_0_breaks.csv'
bpdata = pandas.read_csv(bpfile)
bplst = list(bpdata['gen'])

template = pandas.read_csv(templatepath)
parameters=list(template['var'])
defaults=list(template['defaults'])
mins=list(template['min'])
maxes=list(template['max'])
numparams = len(parameters)

Y,X,T = make_grid(numparams)
print(Y,X,T)

cols=['C0','C1','C2','C3','C4']

fig_size = plt.rcParams["figure.figsize"]
fig_size[0] = 12.8
fig_size[1] = 9.6

plt.rcParams["figure.figsize"] = fig_size
plt.locator_params(nbins=6)

fig, axes = plt.subplots(Y, X, sharex=True)
fig.text(0.5, 0.04, 'Generation', ha='center', size='large')

costs={}

for r in range(len(runs)):
    costs[runs[r]]=[]
    stats = []
    for a in range(numparams):
        stats.append([])
    parafilelst = os.listdir(basepath + runs[r] + paraspath)
    parafilelst.sort()

    parasDF = DataFrame(index=range(len(parafilelst)), columns=parameters)

    for f in parafilelst:
        paras = np.genfromtxt(basepath + runs[r] + paraspath + f)
        minind = np.where(paras[:, -1] == min(paras[:, -1]))
        costs[runs[r]].append(float(paras[minind, -1][0][0]))
        for a in range(numparams):
            stats[a].append(paras[minind][0][a])

    for a in range(numparams):
        axes[T[a][0], T[a][1]].plot(range(len(stats[a])), stats[a], color=cols[r])#, color='grey', ls='--', linewidth=1)
        axes[T[a][0], T[a][1]].plot(bplst[r], stats[a][bplst[r]], color=cols[r],marker='o', zorder=10)
        parasDF[parameters[a]]=stats[a]

    parasDF.to_csv('parameters_'+runs[r]+'.csv')

for a in range(numparams):
    axes[T[a][0],T[a][1]].plot( range(len(costs[runs[0]])),  [defaults[a]]*len(costs[runs[0]]), color='grey', ls='--', linewidth=1)
    axes[T[a][0],T[a][1]].plot( range(len(costs[runs[0]])),  [mins[a]]*len(costs[runs[0]]), color='grey', ls=':', linewidth=1)
    axes[T[a][0],T[a][1]].plot( range(len(costs[runs[0]])),  [maxes[a]]*len(costs[runs[0]]), color='grey', ls=':', linewidth=1)
    axes[T[a][0], T[a][1]].set_title(parameters[a])

legs=[]

for r in range(len(runs)):
    legs.append(mlines.Line2D([], [], color=cols[r], ls='-', label=runs[r]))
legs.append(mlines.Line2D([], [], color='gray', ls='--', label='Default'))
legs.append(mlines.Line2D([], [], color='gray', ls=':', label='Bounds'))
legs.append(mlines.Line2D([], [], color='white', ls=':',marker='o',markerfacecolor='gray', label='Breakpoint'))
leg1=fig.legend(handles=legs, loc='center right', bbox_to_anchor=(1, 0.5))
plt.gca().add_artist(leg1)

for j in range(len(axes[:,0])):
    for i in range(len(axes[0,:])):
        if not axes[j,i].lines:
            fig.delaxes(axes[j][i])

plt.savefig('parameters.png')
plt.cla()

## plot costs

fig_size = plt.rcParams["figure.figsize"]
fig_size[0] = 6.4
fig_size[1] = 4.8
plt.rcParams["figure.figsize"] = fig_size

fig = plt.figure()


costsDF=DataFrame.from_dict(costs,orient='index').transpose()
costsDF.to_csv('costs.csv')

for r in range(len(runs)):
    plt.plot( range(len(costs[runs[r]])), costs[runs[r]],color =cols[r])
    plt.plot(bplst[r], costs[runs[r]][bplst[r]], markerfacecolor=None,markeredgecolor=cols[r], marker='o', zorder=10)

if mode == 'obs':
    defcost = np.genfromtxt(runs[0] + '/parameters/0001.csv')[0,-1]
    plt.plot(range(len(costs[runs[0]])), [defcost]*len(costs[runs[0]]), color='grey', ls='--', linewidth=1)
legs=[]

#legs.append(mlines.Line2D([], [], color='gray', ls='--', label='Default'))
for r in range(len(runs)):
    legs.append(mlines.Line2D([], [], color=cols[r], ls='-', label=runs[r]))
legs.append(mlines.Line2D([], [], color='grey', ls='--', label='Default'))
legs.append(mlines.Line2D([], [], color='white', ls=':',marker='o',markerfacecolor='gray', label='Breakpoint'))
leg1=fig.legend(handles=legs, loc='center right', bbox_to_anchor=(1, 0.5))
plt.gca().add_artist(leg1)
plt.xlabel('Generation')
plt.ylabel('Cost (ST Score)')
plt.savefig('costs.png')

# plot statistics

statnames = ['sPOC', 'bPOC']
for statname in statnames:
    fig, axes = plt.subplots(2, 2, sharex=True)
    fig.text(0.5, 0.04, 'Generation', ha='center', size='medium')
    costs={}

    for r in range(len(runs)):

        filelst=sorted([i for i in os.listdir(basepath + runs[r] + '/stats/') if statname in i])

        allbias=[]
        allstd=[]
        allr=[]
        allrmse=[]
        for f in range(len(filelst)):
            data = pandas.read_csv(basepath + runs[r] + '/stats/' + filelst[f])
            #print(data)
            allbias.append(data['bias'][0])
            allstd.append(data['normstd'][0])
            allr.append(data['R'][0])
            allrmse.append(data['RMSE'][0])

        axes[0,0].plot(allbias, color=cols[r])
        axes[0, 1].plot(allstd, color=cols[r])
        axes[1, 0].plot(allr, color=cols[r])
        axes[1, 1].plot(allrmse, color=cols[r])

        axes[0,0].plot(bplst[r], allbias[bplst[r]], color=cols[r], marker='o', zorder=10)
        axes[0,1].plot(bplst[r], allstd[bplst[r]], color=cols[r], marker='o', zorder=10)
        axes[1,0].plot(bplst[r], allr[bplst[r]], color=cols[r], marker='o', zorder=10)
        axes[1,1].plot(bplst[r], allrmse[bplst[r]], color=cols[r], marker='o', zorder=10)

        # axes[T[a][0], T[a][1]].plot(bplst[r], stats[a][bplst[r]], color=cols[r], marker='o')

        statsDF = DataFrame(index=range(len(filelst)), columns=['bias','normstd','R','RMSE'])
        statsDF['bias']=allbias
        statsDF['normstd']=allstd
        statsDF['R']=allr
        statsDF['RMSE']=allrmse
        statsDF.to_csv(statname + '_stats_' + runs[r] + '.csv')

    legs=[]

    #legs.append(mlines.Line2D([], [], color='gray', ls='--', label='Default'))
    for r in range(len(runs)):
        legs.append(mlines.Line2D([], [], color=cols[r], ls='-', label=runs[r]))
    legs.append(mlines.Line2D([], [], color='grey', ls='--', label='Default'))
    legs.append(mlines.Line2D([], [], color='white', ls=':', marker='o', markerfacecolor='gray', label='Breakpoint'))
    leg1=fig.legend(handles=legs, loc='center right', bbox_to_anchor=(1, 0.5))
    plt.gca().add_artist(leg1)

    axes[ 0, 0].set_title('Bias',size='medium')
    axes[ 0, 1].set_title('Standard Deviation',size='medium')
    axes[ 1, 0].set_title('Correlation',size='medium')
    axes[ 1, 1].set_title('RMSE',size='medium')

    if mode == 'obs':
        defstats = pandas.read_csv(runs[0] + '/stats/'+statname+'0001.csv')
        axes[0, 0].plot(range(len(allbias)), [defstats['bias'][0]] * len(allbias), color='grey', ls='--', linewidth=1)
        axes[0, 1].plot(range(len(allbias)), [defstats['normstd'][0]] * len(allbias), color='grey', ls='--', linewidth=1)
        axes[1, 0].plot(range(len(allbias)), [defstats['R'][0]] * len(allbias), color='grey', ls='--', linewidth=1)
        axes[1, 1].plot(range(len(allbias)), [defstats['RMSE'][0]] * len(allbias), color='grey', ls='--', linewidth=1)

    plt.savefig(statname+'_stats.png')

