from netCDF4 import Dataset
import numpy as np
import os
import pandas as pd

var1 = 'grazflux'
var2 = 'solgoc'
expid = '/09062020/EXPa2wb/'

var1vals = np.genfromtxt(os.getcwd()+expid+var1+'.csv')
var2vals = np.genfromtxt(os.getcwd()+expid+var2+'.csv')

new = Dataset(var1 + '_'+ var2 +'.nc', 'w', format='NETCDF4_CLASSIC')

## create dimensions
new.createDimension('t', 3)
new.createDimension('depth', 3)
new.createDimension(var1, len(var1vals))
new.createDimension(var2, len(var2vals))

new.createVariable('time', np.int32, ('t'))
new.variables['time'][:]=np.array(range(3))
new.createVariable('depth', np.int32, ('t'))
new.variables['depth'][:]=np.array(range(3))
new.createVariable(var1, np.float32, (var1))
new.variables[var1][:]=var1vals
new.createVariable(var2, np.float32, (var2))
new.variables[var2][:]=var2vals

new.createVariable('POC', np.float32, ('t','depth',var1,var2))
new.createVariable('PHY', np.float32, ('t','depth',var1,var2))
new.createVariable('ZOO', np.float32, ('t','depth',var1,var2))
new.createVariable('TPOC', np.float32, ('t','depth',var1,var2))

#new.createVariable('nav_lon', np.float32, ('lat', 'lon'))
shal = 24
medi = 39
deep = 46
dinds = [0,shal,medi,deep]

tinds=[0,24,49,73]

for j in range(len(var2vals)):
    for i in range(len(var1vals)):
        chunk = "{:04d}".format(i+1)
        member = 'fc'+ "{:04d}".format(j)
        inddir = member + '-' + chunk
        piscesout = Dataset(os.getcwd()+expid+ inddir + '/PISCES_5d_19600101_19691231_ptrc_T.nc','r',format='NETCDF4_CLASSIC')

        depths = np.array(piscesout.variables['deptht_bounds'])
        depthdiffs = abs(depths[:,0]-depths[:,1])

        indPOC = np.array(piscesout.variables['POC'])[-72:, :, 1, 1]
        indPHY = np.array(piscesout.variables['PHY'])[-72:, :, 1, 1]
        indZOO = np.array(piscesout.variables['ZOO'])[-72:, :, 1, 1]

        indPOC[indPOC > 10000] = np.nan
        indPHY[indPHY > 10000] = np.nan
        indZOO[indZOO > 10000] = np.nan

        for t in range(3):
            tPOC = np.nanmean(indPOC[tinds[t]:tinds[t + 1], :], axis=0)
            tPHY = np.nanmean(indPHY[tinds[t]:tinds[t + 1], :], axis=0)
            tZOO = np.nanmean(indZOO[tinds[t]:tinds[t + 1], :], axis=0)

            for d in range(3):
                POCweights = []
                PHYweights = []
                ZOOweights = []

                for a in range(dinds[d],dinds[d+1]):
                    print(a)
                    print(len(tPOC))
                    print(len(depthdiffs))
                    # print(sum(depthdiffs))

                    POCweights.append(tPOC[a] * depthdiffs[a] / sum(depthdiffs[dinds[d]:dinds[d+1]]))
                    PHYweights.append(tPHY[a] * depthdiffs[a] / sum(depthdiffs[dinds[d]:dinds[d+1]]))
                    ZOOweights.append(tZOO[a] * depthdiffs[a] / sum(depthdiffs[dinds[d]:dinds[d+1]]))
                print('done')
                print(t,d,i,j)
                new['POC'][t, d, i, j] = sum(POCweights)
                new['PHY'][t, d, i, j] = sum(PHYweights)
                new['ZOO'][t, d, i, j] = sum(ZOOweights)
                print(new['POC'][t, d, i, j] + new['PHY'][t, d, i, j] + new['ZOO'][t, d, i, j])
                new['TPOC'][t, d, i, j] = new['POC'][t, d, i, j] + new['PHY'][t, d, i, j] + new['ZOO'][t, d, i, j]

        # for d in range(3):
        #     for t in range(3):
        #         xPOC = np.nanmean(indPOC[tinds[t]:tinds[t+1], dinds[d]:dinds[d+1]])
        #         xPHY = np.nanmean(indPHY[tinds[t]:tinds[t + 1], dinds[d]:dinds[d + 1]])
        #         xZOO = np.nanmean(indZOO[tinds[t]:tinds[t + 1], dinds[d]:dinds[d + 1]])
        #         new['POC'][t, d, i, j] = xPOC
        #         new['PHY'][t, d, i, j] = xPHY
        #         new['ZOO'][t, d, i, j] = xZOO
        #         new['TPOC'][t, d, i ,j] = xPOC + xPHY + xZOO
        #print(ds)

        piscesout.close()

print(pd.read_csv(os.getcwd() + expid + 'fc0000-0001/indparams.csv'))
Aout = Dataset(os.getcwd() + expid + 'fc0000-0001' + '/PISCES_5d_19600101_19691231_ptrc_T.nc', 'r', format='NETCDF4_CLASSIC')
A = np.array(Aout.variables['ZOO'])[-72:, :, 1, 1]
print(A[0,0])
print(pd.read_csv(os.getcwd() + expid + 'fc0000-0005/indparams.csv'))
Bout = Dataset(os.getcwd() + expid + 'fc0000-0005' + '/PISCES_5d_19600101_19691231_ptrc_T.nc', 'r', format='NETCDF4_CLASSIC')
B = np.array(Bout.variables['ZOO'])[-72:, :, 1, 1]
print(B[0,0])
print(pd.read_csv(os.getcwd() + expid + 'fc0004-0001/indparams.csv'))
Aout = Dataset(os.getcwd() + expid + 'fc0004-0001' + '/PISCES_5d_19600101_19691231_ptrc_T.nc', 'r', format='NETCDF4_CLASSIC')
A = np.array(Aout.variables['ZOO'])[-72:, :, 1, 1]
print(A[0,0])
print(pd.read_csv(os.getcwd() + expid + 'fc0004-0005/indparams.csv'))
Bout = Dataset(os.getcwd() + expid + 'fc0004-0005' + '/PISCES_5d_19600101_19691231_ptrc_T.nc', 'r', format='NETCDF4_CLASSIC')
B = np.array(Bout.variables['ZOO'])[-72:, :, 1, 1]
print(B[0,0])


